// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.

#ifndef ESCWYP_KLAVIER_H
#define ESCWYP_KLAVIER_H
#include <Evas.h>
#include <Elementary.h>
#include "escwyp_globals_extern.h"
#include "escwyp_misch_extern.h"


enum { klavier_max_freq_array_size = 132 };
static int klavier_max_freq_index = klavier_max_freq_array_size-1;
static int klavier_min_freq_index = 0;
static float klavier_max_freq = 1000.0;
static float klavier_min_freq = 0.0;

static unsigned int btn_toggle[4];
static int the_klavier_mode = 0;
static int the_klavier_ignore_mode = 0; 

// the freqs from C-2 through BFLAT-2
const float octave_freqs[12] = {8.175798915643707,8.661957218027252,9.177023997418987,9.722718241315029,10.300861153527185,10.913382232281371,11.562325709738575,12.249857374429665,12.978271799373285,13.75,14.56761754744031,15.433853164253879};
//const float octave_freqs[12] = {440.0,466.1637615180899,493.8833012561241,523.2511306011972,554.3652619537442,587.3295358348151,622.2539674441618,659.2551138257398,698.4564628660078,739.9888454232688,783.9908719634985,830.6093951598903};
float freqs_by_index[klavier_max_freq_array_size];

static char *note_names_lc[120] = {"c0", "c#0", "d0", "d#0", "e0", "f0", "f#0", "g0", "g#0", "a0", "b0", "h0",\
                                 "c1", "c#1", "d1", "d#1", "e1", "f1", "f#1", "g1", "g#1", "a1", "b1", "h1",\
                                 "c2", "c#2", "d2", "d#2", "e2", "f2", "f#2", "g2", "g#2", "a2", "b2", "h2",\
                                 "c3", "c#3", "d3", "d#3", "e3", "f3", "f#3", "g3", "g#3", "a3", "b3", "h3",\
                                 "c4", "c#4", "d4", "d#4", "e4", "f4", "f#4", "g4", "g#4", "a4", "b4", "h4",\
                                 "c5", "c#5", "d5", "d#5", "e5", "f5", "f#5", "g5", "g#5", "a5", "b5", "h5",\
                                 "c6", "c#6", "d6", "d#6", "e6", "f6", "f#6", "g6", "g#6", "a6", "b6", "h6",\
                                 "c7", "c#7", "d7", "d#7", "e7", "f7", "f#7", "g7", "g#7", "a7", "b7", "h7",\
                                 "c8", "c#8", "d8", "d#8", "e8", "f8", "f#8", "g8", "g#8", "a8", "b8", "h8",\
                                 "c9", "c#9", "d9", "d#9", "e9", "f9", "f#9", "g9", "g#9", "a9", "b9", "h9"};
static char *note_names_uc[120] = {"[C0]", "[C#0]", "[D0]", "[D#0]", "[E0]", "[F0]", "[F#0]", "[G0]", "[G#0]", "[A0]", "[B0]", "[H0]",\
                                  "[C1]", "[C#1]", "[D1]", "[D#1]", "[E1]", "[F1]", "[F#1]", "[G1]", "[G#1]", "[A1]", "[B1]", "[H1]",\
                                  "[C2]", "[C#2]", "[D2]", "[D#2]", "[E2]", "[F2]", "[F#2]", "[G2]", "[G#2]", "[A2]", "[B2]", "[H2]",\
                                  "[C3]", "[C#3]", "[D3]", "[D#3]", "[E3]", "[F3]", "[F#3]", "[G3]", "[G#3]", "[A3]", "[B3]", "[H3]",\
                                  "[C4]", "[C#4]", "[D4]", "[D#4]", "[E4]", "[F4]", "[F#4]", "[G4]", "[G#4]", "[A4]", "[B4]", "[H4]",\
                                  "[C5]", "[C#5]", "[D5]", "[D#5]", "[E5]", "[F5]", "[F#5]", "[G5]", "[G#5]", "[A5]", "[B5]", "[H5]",\
                                  "[C6]", "[C#6]", "[D6]", "[D#6]", "[E6]", "[F6]", "[F#6]", "[G6]", "[G#6]", "[A6]", "[B6]", "[H6]",\
                                  "[C7]", "[C#7]", "[D7]", "[D#7]", "[E7]", "[F7]", "[F#7]", "[G7]", "[G#7]", "[A7]", "[B7]", "[H7]",\
                                  "[C8]", "[C#8]", "[D8]", "[D#8]", "[E8]", "[F8]", "[F#8]", "[G8]", "[G#8]", "[A8]", "[B8]", "[H8]",\
                                  "[C9]", "[C#9]", "[D9]", "[D#9]", "[E9]", "[F9]", "[F#9]", "[G9]", "[G#9]", "[A9]", "[B9]", "[H9]"};
static char *img_names[120] = {"c0", "cr", "d0", "cr", "e0", "f0", "cr", "g0", "cr", "a0", "b0", "h0",\
                                 "c1", "cr", "d1", "cr", "e1", "f1", "cr", "g1", "cr", "a1", "b1", "h1",\
                                 "c2", "cr", "d2", "cr", "e2", "f2", "cr", "g2", "cr", "a2", "b2", "h2",\
                                 "c3", "cr", "d3", "cr", "e3", "f3", "cr", "g3", "cr", "a3", "b3", "h3",\
                                 "c4", "cr", "d4", "cr", "e4", "f4", "cr", "g4", "cr", "a4", "b4", "h4",\
                                 "c5", "cr", "d5", "cr", "e5", "f5", "cr", "g5", "cr", "a5", "b5", "h5",\
                                 "c6", "cr", "d6", "cr", "e6", "f6", "cr", "g6", "cr", "a6", "b6", "h6",\
                                 "c7", "cr", "d7", "cr", "e7", "f7", "cr", "g7", "cr", "a7", "b7", "h7",\
                                 "c8", "cr", "d8", "cr", "e8", "f8", "cr", "g8", "cr", "a8", "b8", "h8",\
                                 "c9", "cr", "d9", "cr", "e9", "f9", "cr", "g9", "cr", "a9", "b9", "h9"};
static char *img_names_a[120] = {"c0a", "cra", "d0a", "cra", "e0a", "f0a", "cra", "g0a", "cra", "a0a", "b0a", "h0a",\
                                 "c1a", "cra", "d1a", "cra", "e1a", "f1a", "cra", "g1a", "cra", "a1a", "b1a", "h1a",\
                                 "c2a", "cra", "d2a", "cra", "e2a", "f2a", "cra", "g2a", "cra", "a2a", "b2a", "h2a",\
                                 "c3a", "cra", "d3a", "cra", "e3a", "f3a", "cra", "g3a", "cra", "a3a", "b3a", "h3a",\
                                 "c4a", "cra", "d4a", "cra", "e4a", "f4a", "cra", "g4a", "cra", "a4a", "b4a", "h4a",\
                                 "c5a", "cra", "d5a", "cra", "e5a", "f5a", "cra", "g5a", "cra", "a5a", "b5a", "h5a",\
                                 "c6a", "cra", "d6a", "cra", "e6a", "f6a", "cra", "g6a", "cra", "a6a", "b6a", "h6a",\
                                 "c7a", "cra", "d7a", "cra", "e7a", "f7a", "cra", "g7a", "cra", "a7a", "b7a", "h7a",\
                                 "c8a", "cra", "d8a", "cra", "e8a", "f8a", "cra", "g8a", "cra", "a8a", "b8a", "h8a",\
                                 "c9a", "cra", "d9a", "cra", "e9a", "f9a", "cra", "g9a", "cra", "a9a", "b9a", "h9a"};
static char *img_path;
static char *img_suffix = ".png";

Evas_Object *klavier_box;
Evas_Object *btn_mul2;
static Evas_Object *klavier_tasten_table; 

static Evas_Object *klavier_mode_check;
static Evas_Object *klavier_ignore_freq_check;

static Evas_Object *the_btns[klavier_max_freq_array_size];
static Evas_Object *the_imgs[klavier_max_freq_array_size];

static Evas_Object *img_taste[klavier_max_freq_array_size];
static Evas *canv;
static unsigned int *pixels[klavier_max_freq_array_size];
static int tasten_x = 16;
static int tasten_y = 16;
static int tasten_argb_fg[2];
static int tasten_argb_bg[2];
static unsigned char tasten_type[klavier_max_freq_array_size];
 
int klavierInitGui();
void klavier_init_keygrabs();
void klavierGuiPostbox(Evas_Object **obj_arr);
float klavierGetMaxFreq();
int klavierGetMaxFreqIndex();
float klavierGetMinFreq();
int klavierGetMinFreqIndex();
int klavierGetIgnoreMode();
void init_freqs();
static void clear_all_btns();
static void klavier_mode_check_fun(void *data, Evas_Object *obj, void *event_info);
static void klavier_ignore_freq_check_fun(void *data, Evas_Object *obj, void *event_info);
static void btn_gen_fun(void *data, Evas_Object *obj, void *event_info);
static int btn_tgl_fun(const int id);
static int btn_tgl_query(const int id);
static void btn_release_fun(void *data, Evas_Object *obj, void *event_info);
static void tastbild_fun(void *data, Evas_Object *obj, void *event_info);
static void handle_key_up(void *data EINA_UNUSED, Evas *evas, Evas_Object *o EINA_UNUSED, void *einfo);
static void handle_key_down(void *data EINA_UNUSED, Evas *evas, Evas_Object *o EINA_UNUSED, void *einfo);

static void btn_mul_down_fun(void *data, Evas_Object *obj, const char *emission, const char *source);
static void btn_mul_up_fun(void *data, Evas_Object *obj, const char *emission, const char *source);
static void btn_mul_notone_fun_on(void *data, Evas_Object *obj, const char *emission, const char *source);
static void btn_mul_notone_off_fun(void *data, Evas_Object *obj, const char *emission, const char *source);

#endif
