// Copyright 2012-2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <string.h>
#include "escwyp_main.h"
#include "escwyp_globals.h"
#include "escwyp_chain_gui_elm_extern.h"
#include "escwyp_sine_robin_gui_elm_extern.h"
#include "escwyp_sine_grass_gui_elm_extern.h"
#include "escwyp_nadelper_gui_elm_extern.h"
#include "escwyp_samplecast_gui_elm_extern.h"
#include "escwyp_samplecast_extern.h"
#include "escwyp_spratz_gui_elm_extern.h"
#include "escwyp_spratz_extern.h"
#include "escwyp_main_gui_elm_extern.h"
#include "escwyp_out_txt_extern.h"
#include "escwyp_out_none_extern.h"
#include "escwyp_misch_extern.h"
#include "escwyp_oszillo_extern.h"
#include "escwyp_klavier_extern.h"
#ifdef ESCWYP_OUT_IS_ALSA
    #include "escwyp_out_alsa_extern.h"
#endif
#ifdef ESCWYP_OUT_IS_JACK
    #include "escwyp_out_jack_extern.h"
#endif
#include "escwyp_terminal.h"

void create_buffers(const int length)
{

       int n = 0;

       if(wartemal){
	   free(wartemal);
	   fprintf(stdout, "wartemal freed.\n");
       }
       wartemal = malloc(length*sizeof(sample_t));
       fprintf(stdout, "WARTEMAL: %x\n", wartemal);
       for(n = 0; n < length; n++){
           wartemal[n] = 0.0;
       }
       // die 4: (byte)-size of sample_t, die 8: acht mal jack_buffer_length, die length: jack_bufer_length 
       if(buff_L){
	   free(buff_L);
	   fprintf(stdout, "buff_L freed.\n");
       }
       buff_L = malloc(length * buff_parts * sizeof(sample_t));
       fprintf(stdout, "buff_L Start-Adr: %p, End-Adr: %p %i\n", buff_L, buff_L + length*buff_parts, sizeof(sample_t));

       // sinranbuf enthält length Werte der Sinusfunktion wischen 0 und 2*PI, positiviert, d.h. +1 und geteilt durch 2
       if(sinranbuf ){
	   free(sinranbuf );
	   fprintf(stdout, "sinranbuf freed.\n");
       }
       sinranbuf = malloc(length * sizeof(double));
       for(n = 0; n < length; n++){
           sinranbuf[n] = (sin(n*2.0*M_PI/(length-1)) + 1) / 2;
       }

       // x8ranbuf enthält length Werte der Funktion x^8 wischen 0 und 1
       if(x8ranbuf ){
	   free(x8ranbuf );
	   fprintf(stdout, "x8ranbuf freed.\n");
       }
       x8ranbuf = malloc(length * sizeof(double));
       for(n = 0; n < length; n++){
           x8ranbuf[n] = pow(( (double) n/(length-1)), 8.0);
       }

       // sin28ranbuf enthält length Werte der Funktion (sin(28*x)+4)/8 wischen 0 und 1
       if(sin28ranbuf ){
	   free(sin28ranbuf );
	   fprintf(stdout, "sin28ranbuf freed.\n");
       }
       sin28ranbuf = malloc(length * sizeof(double));
       for(n = 0; n < length; n++){
           sin28ranbuf[n] = (sin( 28.0 * n/(length-1))+48) / 96.0;
           //fprintf(stdout, "sin28ranbuf: %f\n", sin28ranbuf[n]);
       }

       // the Envelope to lead aout the sound aufter the key/button is released
       if(lead_out_envelope ){
	   free(lead_out_envelope );
	   fprintf(stdout, "lead_out_envelope freed.\n");
       }
       lead_out_envelope = malloc(length*sizeof(sample_t));
       for(n = 0; n < length; n++){
           lead_out_envelope[n] = (sample_t) 1.0 - (1.0*n)/(length-1);
       }
       
       sound_buffer_length = length;
       misch_init_buffer(sound_buffer_length);

}

void change_play_status(const int method)
{

   playsth = method;

}


void release_klavier_control(const int value)
{

	chain_gui_set_self_control(value);
	sine_robin_gui_set_self_control(value);
	sine_grass_gui_set_self_control(value);
	nadelper_gui_set_self_control(value);
	samplecast_gui_set_self_control(value);
	spratz_gui_set_self_control(value);

}

int escwyp_switch_backend(const int val)
{
   misch_save_num_nts();
   if(out_m == jack){
#ifdef ESCWYP_OUT_IS_JACK
      escwyp_jack_shutdown_wrapper();
#endif
   }
   if(out_m == alsa){ 
#ifdef ESCWYP_OUT_IS_ALSA
      escwyp_out_alsa_stop();
#endif
   }
   if(out_m == none){ 
      escwyp_out_none_stop();
   }

   int init_ret = -1;
#ifdef ESCWYP_OUT_IS_JACK
   if(val == 0){
      init_ret = escwypInitJack();
      if(init_ret == 0){
         out_m = jack;
         fprintf(stdout, "Switched to Jack\n");
      }
   }
#endif
#ifdef ESCWYP_OUT_IS_ALSA
   if(val == 1){
      //sound_buffer_length = 4096;
      init_ret = escwyp_out_alsa_init();
      if(init_ret == 0){
         out_m = alsa;
         fprintf(stdout, "Switched to Alsa\n");
      }
   }
#endif
   if(val == 2){
      init_ret = escwyp_out_none_init();
      if(init_ret == 0){
         out_m = none;
         fprintf(stdout, "Switched to None\n");
      }
   }
   if(init_ret != 0){
      if(out_m == jack){
#ifdef ESCWYP_OUT_IS_JACK
         escwypInitJack();
#endif
      }
      if(out_m == alsa){ 
#ifdef ESCWYP_OUT_IS_ALSA
         escwyp_out_alsa_init();
#endif
      }
      if(out_m == none){ 
         escwyp_out_none_init();
      }
   }
   misch_restore_num_nts();

   return out_m;
}

void shutdown_stuff()
{

   if(out_m == jack){
#ifdef ESCWYP_OUT_IS_JACK
      escwyp_jack_shutdown_wrapper();
#endif
   }
   if(out_m == alsa){ 
#ifdef ESCWYP_OUT_IS_ALSA
      escwyp_out_alsa_stop();
#endif
   }
   if(out_m == none){ 
      escwyp_out_none_stop();
   }

}

int escwyp_switch_method(const int val)
{

   if(val > 0){
       current_sound_function = val+2;
   }else{ 
       current_sound_function = val+1;
   }
   escwyp_migrate_notes();

   return 0;

}


EAPI_MAIN int elm_main(int argc, char **argv)
{

   fprintf(stdout, "A R G S = %i\n", argc);
   #ifdef ESCWYP_OUT_IS_ALSA
   out_m = alsa;
   #endif
   #ifdef ESCWYP_OUT_IS_JACK
   out_m = jack;
   #endif
   enum interface_type ui_type = elementary;

   //create_buffers(8192);

   int list_opt_mode = 0;
   if(argc > 1){
       char *out_method_opt = argv[1];
#ifdef ESCWYP_OUT_IS_ALSA
       if( strcmp(out_method_opt, "aoa") == 0){
           out_m = alsa;
       }
       if( strcmp(out_method_opt, "aoat") == 0){
           out_m = alsa;
           ui_type = terminal;
       }
#endif
#ifdef ESCWYP_OUT_IS_JACK
       if( strcmp(out_method_opt, "aoj") == 0){
           out_m = jack;
       }
       if( strcmp(out_method_opt, "aojt") == 0){
           out_m = jack;
           ui_type = terminal;
       }
#endif
       if( strcmp(out_method_opt, "aon") == 0){
           out_m = none;
       }
       if( strcmp(out_method_opt, "aont") == 0){
           out_m = none;
           ui_type = terminal;
       }
       if( strcmp(out_method_opt, "aolist") == 0){
            list_opt_mode = 1;
#ifdef ESCWYP_OUT_IS_JACK
            fprintf(stdout, "call ./escwyp aoj\n\n for Jack output, and Elemnetary GUI\n\n");
            fprintf(stdout, "call ./escwyp aojt\n\n for Jack output, and terminal interface\n\n");
#endif
#ifdef ESCWYP_OUT_IS_ALSA
            fprintf(stdout, "call ./escwyp aoa\n\n for ALSA output, and Elemnetary GUI\n\n");
            fprintf(stdout, "call ./escwyp aoat\n\n for ALSA output, and terminal interface\n\n");
#endif
            fprintf(stdout, "call ./escwyp aon\n\n for no audio output, and Elemnetary GUI\n\n");
            fprintf(stdout, "call ./escwyp aont\n\n for no audio output, and terminal interface\n\n");
       }

   }
   if(!list_opt_mode){ 


       srandom(time(0));
       misch_init();
       oszilloInitBuffer(1024);
       initTxtOut();

       int backend_start_success = 0;
#ifdef ESCWYP_OUT_IS_JACK
       // JACK //
       if(out_m == jack){
           if(escwypInitJack() == 0){
               fprintf(stdout, "Starting JACK out\n");
	       backend_start_success = 1;
           }else{
               fprintf(stdout, "Failed to start JACK out\n");
           }
       }
       // ---- //
#endif
       // ALSA //

#ifdef ESCWYP_OUT_IS_ALSA
       if(out_m == alsa){	       
           if(escwyp_out_alsa_init() == 0){
               fprintf(stdout, "Starting ALSA out\n");
	       backend_start_success = 1;
           }else{
               fprintf(stdout, "Failed to start ALSA out\n");
           }
       }
       // ---- //
#endif
       if(out_m == none){
           if(escwyp_out_none_init() == 0){
               fprintf(stdout, "Starting none out\n");
	       backend_start_success = 1;
           }else{
               fprintf(stdout, "Failed to start none out\n");
           }
       }
       if(backend_start_success == 0){
#ifdef ESCWYP_OUT_IS_JACK
          // JACK //
          if(out_m != jack){
              if(escwypInitJack() == 0){
                  fprintf(stdout, "Starting alternatively JACK out\n");
                  backend_start_success = 1;
                  out_m = jack;
              }else{
                  fprintf(stdout, "Failed to alternatively start JACK out\n");
              }
          }
       // ---- //
#endif
       // ALSA //

#ifdef ESCWYP_OUT_IS_ALSA
          if(out_m != alsa && backend_start_success == 0){	       
              if(escwyp_out_alsa_init() == 0){
                  fprintf(stdout, "Starting alternatively ALSA out\n");
                  backend_start_success = 1;
                  out_m = alsa;
              }else{
                  fprintf(stdout, "Failed to alternatively start ALSA out\n");
              }
          }
          // ---- //
#endif
          if(out_m != none && backend_start_success == 0){
              if(escwyp_out_none_init() == 0){
                  fprintf(stdout, "Starting alternatively none out\n");
                  backend_start_success = 1;
                  out_m = none;
              }else{
                  fprintf(stdout, "Failed to alternatively start none out\n");
              }
          }
       }

       fprintf(stdout, "A U D I O  %i  S T A R T E T\n", out_m);
       current_sound_function = 1;
       //from the klavier
       init_freqs();
       samplecastConstructor();
       spratz_constructor();
       oszilloInitSem();
       playsth = 0;
       // ----------------------
       //misch_init(sound_buffer_length); // nach escwyp_main.c
       if(ui_type == elementary){
          // run the mainloop and process events and callbacks
          // MAKE THE WINDOW ------   
          escwypMainGuiInit();
          elm_run();
          elm_shutdown();
       }

       if(ui_type == terminal){
          escwyp_terminal_menu();
       }
   }

   return 0;
}
ELM_MAIN()
