// Copyright 2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version. See http://www.gnu.org/licenses/

#ifndef ESCWYP_OUT_TXT_H
#define ESCWYP_OUT_TXT_H
#include <stdio.h>
#include <semaphore.h>

static sem_t sem;

struct buffpart{
   float *b; //Der Buffer
   int bf;   //Fuellgrad des Buffers als #floats
   sem_t s;  //Das Semaphor
};

static const int buffs_num = 2;
static struct buffpart buffs[2];
static int buffs_index = 0;
static int write_index = 0;

// if 1, the sound buffer will be put to TXT.
// if 0, not
static int output_txt = 0;
static int output_txt_offarm = 0;
static int output_txt_onarm = 0;
static int output_txt_notes = 0;

// counter for counting how many sound buffers have already been output as TXT
int sound_text_counter = 0;

// the filename/path for the TXT file
static char filenamebase[30] = "/tmp/";
static char filename[300] = "";
static char filenameraw[30] = "";
static char txt_out_status[100] = "";
static int txt_out_result = 1;
static int wav_out_result = 1;
static void *sndfile = NULL;


void initTxtOut();
void outTxtCopy(const int length, const float *buff);
static int init_write_thread(int index);
static void *processTxt(void *args);
static void outputSoundAsText(const int length, float *buff);
#ifdef ESCWYP_SNDFILE_YES
static void outputSoundAsWav(const int length, float *buff);
#endif
int get_output_txt();
static int set_output_txt(const int val);

// close the file
static void afterSoundAsText();

// buffer for storing the floats from the sound buffer
static float *txt_buff;

// index pointing to the region in 'txt_buff' that will be written next
static int txt_buff_index = 0;

// Percentage off buffer fill
static double txt_buff_fill = 0.0;

// the length of txt_buff
static int txt_buff_length = 48000*5;

// the file corresponding to 'filename'
static FILE *outfile; 

void outTxtRecOnArm(const int val); //Call this to start recording after outTxtNotes() has been called
void outTxtRec(); //Call this to start recording now
void outTxtStop(); //Call this to stop recording or cancel an arming with outTxtRecOnArm()
void outTxtRecOffArm(const int val); //Call this to stop recording once outTxtSilence() has been called
void outTxtNotes(); //Call this to notify out_txt that notes just startet playing
void outTxtSilence(); //Call this to notify out_txt that notes have ceased and we have silence
void printstatus();
int get_output_txt_status();
int get_output_wav_status();
char *get_output_filename();
static void yellstatus();

void (*escwyp_rec_progress_cb)(const float f);
void (*escwyp_rec_lab_cb)(const int i, const char *c);
void (*escwyp_rec_btns_cb)(void);


#endif
