// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include <Elementary.h>
#include "escwyp_globals_extern.h"
#include "escwyp_rec_gui_elm.h"
#include "escwyp_out_txt_extern.h"
#include <string.h>

void recPackGui(Evas_Object *box, Evas_Object *box2){

    elm_box_pack_end(box, btn_rec);
    evas_object_show(btn_rec);

    elm_box_pack_end(box, chk_aton_rec);
    evas_object_show(chk_aton_rec);

    elm_box_pack_end(box, bar_rec);
    evas_object_show(bar_rec);

    elm_box_pack_end(box, chk_atoff_rec);
    evas_object_show(chk_atoff_rec);

    elm_box_pack_end(box2, lab_rec);
    evas_object_show(lab_rec);

}

void recUnpackGui(Evas_Object *box, Evas_Object *box2){

    elm_box_unpack(box, btn_rec);
    evas_object_hide(btn_rec);

    elm_box_unpack(box, chk_aton_rec);
    evas_object_hide(chk_aton_rec);

    elm_box_unpack(box, bar_rec);
    evas_object_hide(bar_rec);

    elm_box_unpack(box, chk_atoff_rec);
    evas_object_hide(chk_atoff_rec);

    elm_box_unpack(box2, lab_rec);
    evas_object_hide(lab_rec);

}

void recInitGuiElm(){

   // the Record Button
   btn_rec = elm_button_add(win);
   elm_object_text_set(btn_rec, "REC");
   evas_object_show(btn_rec);
   evas_object_smart_callback_add(btn_rec, "clicked", btn_rec_fun, NULL);


   chk_aton_rec = elm_check_add(win);
   elm_object_part_text_set(chk_aton_rec, NULL, "@ON");
   elm_check_state_set(chk_aton_rec, EINA_FALSE); 
   evas_object_smart_callback_add(chk_aton_rec, "changed", chk_aton_rec_fun, NULL);
   evas_object_show(chk_aton_rec);

   bar_rec = elm_progressbar_add(win);
   elm_progressbar_span_size_set(bar_rec, 200);
   evas_object_show(bar_rec);

   chk_atoff_rec = elm_check_add(win);
   elm_object_part_text_set(chk_atoff_rec, NULL, "@OFF");
   elm_check_state_set(chk_atoff_rec, EINA_FALSE); 
   evas_object_smart_callback_add(chk_atoff_rec, "changed", chk_atoff_rec_fun, NULL);
   evas_object_show(chk_atoff_rec);
   
   lab_rec = elm_label_add(win);
   elm_object_text_set(lab_rec, "Cool");
   evas_object_show(lab_rec);

   sem_init(&sem_bar, 0, 1);
   sem_init(&sem_lab, 0, 1);
   int semv = 0;
   sem_getvalue(&sem_bar, &semv);
   fprintf(stdout, "Semaphore sem_bar of rec_gui %i\n", semv);
   sem_getvalue(&sem_lab, &semv);
   fprintf(stdout, "Semaphore sem_lab of rec_gui %i\n", semv);

   elm_object_text_set(lab_rec, "1");
   elm_object_text_set(lab_rec, "2");
   elm_object_text_set(lab_rec, "3");
   elm_object_text_set(lab_rec, "4");
   elm_object_text_set(lab_rec, "5");
   elm_object_text_set(lab_rec, "6");
   elm_object_text_set(lab_rec, "7");
   elm_object_text_set(lab_rec, "8");
   elm_object_text_set(lab_rec, "9");

   escwyp_rec_progress_cb = bar_rec_fun;
   escwyp_rec_lab_cb = lab_rec_fun;
   escwyp_rec_btns_cb = btns_rec_fun;

   rec_status_array = malloc(sizeof(struct rec_status)*rec_status_array_size);
   int i = 0;
   for(i = 0; i < rec_status_array_size; ++i){
      rec_status_array[i].text = malloc(sizeof(char)*rec_status_text_size);
      rec_status_array[i].status = 0;
   }

   rec_status_timer = ecore_timer_add(0.1, rec_status_func, NULL);

}

static void btn_rec_fun(void *data, Evas_Object *obj, void *event_info){

   if(btn_rec_state == 0){
      outTxtRec(); //Call this to start recording now
      btn_rec_state = 1;
      elm_object_text_set(btn_rec, "A/S");
      recHideChks();
   }else{
      if(btn_rec_state == 1){
         outTxtStop(); //Call this to stop recording or cancel an arming with outTxtRecOnArm()
         btn_rec_state = 0;
         elm_object_text_set(btn_rec, "REC");
         recShowChks();
      }
   }
   fprintf(stdout, "Yis btn_rec_fun STATE %i\n", btn_rec_state);

}

void bar_rec_fun(const float val)
{
   int sr = -1;
   int se = -1;
   sr = sem_trywait(&sem_bar);
   se = errno;
   if(sr == 0){ 
      elm_progressbar_value_set(bar_rec, (double)val);
      sem_post(&sem_bar); 
   }
}
static void chk_aton_rec_fun(void *data, Evas_Object *obj, void *event_info)
{

   int val = elm_check_state_get(chk_aton_rec);
   outTxtRecOnArm(val); //Call this to start recording after outTxtNotes() has been called
   fprintf(stdout, "Yis chk_aton_rec_fun STATE %i\n", val);
}
static void chk_atoff_rec_fun(void *data, Evas_Object *obj, void *event_info)
{
   int val = elm_check_state_get(chk_atoff_rec);
   outTxtRecOffArm(val); //Call this to stop recording once outTxtSilence() has been called
   fprintf(stdout, "Yis chk_atoff_rec_fun STATE %i\n", val);

}
void lab_rec_fun(const int state, const char *str)
{
   int sr = -1;
   int se = -1;
   sr = sem_trywait(&sem_lab);
   se = errno;
   if(sr == 0){ 
      strncpy(lab_rec_str, str, 100);
      //fprintf(stdout, "+ + + + lab_rec_fun %s\n", lab_rec_str);
      if(state == 1){
         elm_object_text_set(lab_rec, "Pending");
      }else{
         elm_object_text_set(lab_rec, lab_rec_str); //TODO: If this is in the code it will segfault on high frequency calls
      }
      sem_post(&sem_lab); 
   }
   fprintf(stdout, "Yis lab_rec_fun %i\n", state);

}
void recHideBtn()
{

    Eina_Bool eb = elm_object_disabled_get(btn_rec);
    if(eb == EINA_FALSE){
       elm_object_disabled_set(btn_rec, EINA_TRUE);
    }
    //evas_object_hide(btn_rec);

}
void recShowBtn()
{

    Eina_Bool eb = elm_object_disabled_get(btn_rec);
    if(eb == EINA_TRUE){
       elm_object_disabled_set(btn_rec, EINA_FALSE);
    }
    //evas_object_show(btn_rec);
    btn_rec_state = 0;
    elm_object_text_set(btn_rec, "REC");

}
void recHideChks()
{
   elm_object_disabled_set(chk_aton_rec, EINA_TRUE);
   elm_object_disabled_set(chk_atoff_rec, EINA_TRUE);
}
void recShowChks()
{
   elm_object_disabled_set(chk_aton_rec, EINA_FALSE);
   elm_object_disabled_set(chk_atoff_rec, EINA_FALSE);
   int val = elm_check_state_get(chk_aton_rec);
   elm_check_state_set(chk_aton_rec, !val);
   elm_check_state_set(chk_aton_rec, val);
   val = elm_check_state_get(chk_atoff_rec);
   elm_check_state_set(chk_atoff_rec, !val);
   elm_check_state_set(chk_atoff_rec, val);
}

void btns_rec_fun()
{

   recShowBtn();
   recShowChks();

}

int update_rec_status(const char *c, const int size, const int status)
{
   if(status == 0){
      /* recording successful */
      rec_status_array[rec_status_new_index].status = status;
   }else if(status == 1){
      /* recording failed */
      rec_status_array[rec_status_new_index].status = status;
   }else if(status == 2){
      /* recording running */
      if(strncmp(rec_status_array[rec_status_new_index].text, c, size) != 0){
         ++rec_status_new_index >= rec_status_array_size ? rec_status_new_index = 0 : rec_status_new_index; 
         strncpy(rec_status_array[rec_status_new_index].text, c, rec_status_text_size); 
         rec_status_array[rec_status_new_index].text[rec_status_text_size-1] = '\0';
         rec_status_array[rec_status_new_index].status = status;
      }
   }
}

static Eina_Bool rec_status_func(void *data) {

   //elm_object_text_set(lab_rec, rec_status_array[rec_status_new_index].text);
   /* int txt_status = get_output_txt_status();
   int wav_status = get_output_wav_status();
   char *filename = get_output_filename(); */
   return ECORE_CALLBACK_RENEW;
   //ECORE_CALLBACK_CANCEL
}

