// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include "escwyp_sine_robin_patch_eet.h"
#include "escwyp_sine_robin_extern.h"
#include "escwyp_sine_robin_gui_elm_extern.h"
#include "escwyp_patch_select_extern.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void sine_robin_load_patch(const char *patch_name){

    fprintf(stdout, "III sine_robin_load_patch\n");

    void *param_p = NULL;
    void *param_pp = NULL;
    char the_base_key[255];
    char the_key[255];
    the_base_key[0] = 47;
    the_base_key[1] = 0;
    strcat(the_base_key, "sine_robin/"); 
    strcat(the_base_key, patch_name);

    // DISTRIB CENTER----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_center");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
       float *distrib_center = (float *) param_pp;
       sine_robin_set_distrib_center(*distrib_center);
       sine_robin_center_slider_set(*distrib_center);

       fprintf(stdout, "%s %f\n", the_key, *distrib_center);
       free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // DISTRIB HEIGHT----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_height");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
        float *distrib_height = (float *) param_pp;
        sine_robin_set_distrib_height(*distrib_height);
        sine_robin_height_slider_set(*distrib_height);

        fprintf(stdout, "%s %f\n", the_key, *distrib_height);
        free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // DISTRIB HEIGHT FINE FRAC----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_height_fine_frac");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
        float *distrib_height_fine_frac = (float *) param_pp;
        sine_robin_set_distrib_height_fine_frac(*distrib_height_fine_frac);
        sine_robin_height_fine_frac_slider_set(*distrib_height_fine_frac);

        fprintf(stdout, "%s %f\n", the_key, *distrib_height_fine_frac);
        free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // DISTRIB LOW----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_low");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
        int *distrib_low = (int *) param_pp;
        sine_robin_set_distrib_low(*distrib_low);
        sine_robin_low_slider_set(*distrib_low);

        fprintf(stdout, "%s %i\n", the_key, *distrib_low);
        free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // DISTRIB HIGH----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_high");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
        int *distrib_high = (int *) param_pp;
        sine_robin_set_distrib_high(*distrib_high);
        sine_robin_high_slider_set(*distrib_high);

        fprintf(stdout, "%s %i\n", the_key, *distrib_high);
        free(param_pp);
    }else{
        fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // DISTRIB FUNCTION----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/distrib_function");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
        int *distrib_function = (int *) param_pp;
        sine_robin_set_distrib_function(*distrib_function);
        sine_robin_radio_0_set(*distrib_function);

        fprintf(stdout, "%s %i\n", the_key, *distrib_function);
        free(param_pp);
        sine_robin_update_distrib();
        makeDistribImage();
    }else{
        fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

}

void sine_robin_save_patch(const char *patch_name){

	fprintf(stdout, "sine_robin_save_patch\n");
	float distrib_center = 0.0;
	float distrib_height = 0.0;
	float distrib_height_fine_frac = 0.0;
	int distrib_low = 0;
	int distrib_high = 0;
	int distrib_function = 0;

	char id_string[255];
	strcpy(id_string, "/PATCH_NAME/sine_robin/");
	strcat(id_string, patch_name);
	escwyp_patch_save_param(patch_name, (1+strlen(patch_name))*sizeof(char), id_string);
	fprintf(stdout, "%s = %s\n", id_string, patch_name);

	char the_base_key[255];
	char the_key[255];
	the_base_key[0] = 47;
	the_base_key[1] = 0;
	strcat(the_base_key, "sine_robin/"); 
	strcat(the_base_key, patch_name);


	distrib_center = sine_robin_get_distrib_center();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_center");
	escwyp_patch_save_param(&distrib_center, sizeof(float), the_key);
	fprintf(stdout, "%s %f\n", the_key, distrib_center);

	distrib_height = sine_robin_get_distrib_height();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_height");
	escwyp_patch_save_param(&distrib_height, sizeof(float), the_key);
	fprintf(stdout, "%s %f\n", the_key, distrib_height);

	distrib_height_fine_frac = sine_robin_get_distrib_height_fine_frac();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_height_fine_frac");
	escwyp_patch_save_param(&distrib_height_fine_frac, sizeof(float), the_key);
	fprintf(stdout, "%s %f\n", the_key, distrib_height_fine_frac);

	distrib_low = sine_robin_get_distrib_low();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_low");
	escwyp_patch_save_param(&distrib_low, sizeof(int), the_key);
	fprintf(stdout, "%s %i\n", the_key, distrib_low);

	distrib_high = sine_robin_get_distrib_high();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_high");
	escwyp_patch_save_param(&distrib_high, sizeof(int), the_key);
	fprintf(stdout, "%s %i\n", the_key, distrib_high);

	distrib_function = sine_robin_get_distrib_function();
	strcpy(the_key, the_base_key);
	strcat(the_key, "/distrib_function");
	escwyp_patch_save_param(&distrib_function, sizeof(int), the_key);
	fprintf(stdout, "%s %i\n", the_key, distrib_function);


}
