// Copyright 2014-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

//#include "escwyp_globals_extern.h"
#include "escwyp_spratz.h"
#include <math.h>
#include <stdio.h>

void spratz_constructor()
{

    //x_fact = 0.475;
    //y_fact = 3.793; // ! -gt zero && != 1.0
    //x_fact = 1.0;
    //y_fact = 2.0; // ! -gt zero
    y_fact_inv = 1/y_fact;

    max_height = powf(y_fact, depth-1);
    //x_length = x_fact*(sqrt(powf( (max_height-1)/cosf(atanf( 1/(y_fact-1) )) , 2) - powf((max_height-1), 2))*2+max_height);
    x_length = x_fact*(2*(max_height-1)/(y_fact-1) + max_height);
    quer_length = sqrt(pow(max_height, 2) + pow(x_length, 2));

    //sample_rate = 1000.0;
    //sample_width = 1/sample_rate;

    // Fuer die Rotation des Spratz hier den Winkel etc. ausrechnen
    verhaeltnis = max_height/x_length;
    winkerl = atanf(verhaeltnis); // note the r in winkerl !!
    rot_0_0 = cosf(winkerl); 
    rot_0_1 = sinf(winkerl); 
    rot_1_0 = -rot_0_1; // -sinf(winkerl) 
    rot_1_1 = rot_0_0;  //  cosf(winkerl) 

    //fprintf(stdout, "# max_height = %f, x_length = %f, x_length = %f, quer_length = %f, quer_length*sample_width = %f, x_fact = %f, y_fact = %f, sample_rate = %f, sample_width = %f, depth = %i, verhaeltnis = %f, winkerl = %f, (%f %f %f %f) \n", max_height, x_length, x_length, quer_length, quer_length*sample_width, x_fact, y_fact, sample_rate, sample_width, depth, verhaeltnis, winkerl, rot_0_0, rot_0_1, rot_1_0, rot_1_1);
    int i;
    for(i = 0; i < 4; ++i){
        spratz_shadow_vals[i] = 0.0;
        spratz_shadow_vals_changed[i] = 0;
    }
}

void spratz_refresh_parameters()
{

    y_fact_inv = 1/y_fact;

    max_height = powf(y_fact, depth-1);
    //x_length = x_fact*(sqrt(powf( (max_height-1)/cosf(atanf( 1/(y_fact-1) )) , 2) - powf((max_height-1), 2))*2+max_height);
    x_length = x_fact*(2*(max_height-1)/(y_fact-1) + max_height);
    quer_length = sqrt(pow(max_height, 2) + pow(x_length, 2));

    //sample_rate = 1000.0;
    //sample_width = 1/sample_rate;

    // Für die Rotation des Spratz hier den Winkel etc. ausrechnen
    verhaeltnis = max_height/x_length;
    winkerl = atanf(verhaeltnis); // note the r in winkerl !!
    rot_0_0 = cosf(winkerl); 
    rot_0_1 = sinf(winkerl); 
    rot_1_0 = -rot_0_1; // -sinf(winkerl) 
    rot_1_1 = rot_0_0;  //  cosf(winkerl) 

    if(the_params->degraded != degrade_id){
        fprintf(stdout, "NEED UPDATE id=%i, degraded=%i\n", degrade_id, the_params->degraded);
        float freq_save = the_params->sample_rate;
        spratz_init_values();
        spratz_set_freq(freq_save);
        the_params->degraded = degrade_id;
    }
}

void spratz_set_params(struct spratz_params *params)
{

    the_params = params;

}

void spratz_set_freq(const float freq)
{

    the_params->sample_rate = freq;
    fprintf(stdout, " ----------------- SPRATZ_SAMPLERATE = %f\n", the_params->sample_rate);

}

void spratz_set_freq_from_klavier(const float freq, const float freq_low, const float freq_high)
{

    //float the_low = 100.0;
    //float the_high = 14.0;
    //float range = freq_high - freq_low;
    //if(range > 0.0){

    //    the_params->sample_rate = the_low + (the_high - the_low)*(freq - freq_low)/range;

    //}
    if(!self_control){
        if(freq_high-freq_low > 0.0){
            float tmp_rate = freq_high*7.0/freq;
            if(tmp_rate >= 7.0 ){
                the_params->sample_rate = tmp_rate;
            }
        }
    }else{
        the_params->sample_rate = spratz_sample_rate;
    }
    fprintf(stdout, " ----------------- SPRATZ_SAMPLERATE = %f, freq_low = %f, freq_high = %f\n", the_params->sample_rate, freq_low, freq_high);

}

void spratz_init_values()
{

    the_params->the_points[0] = 0.0;
    the_params->the_points[1] = 0.0;
    the_params->the_offsets[0] = 0.0;
    the_params->the_offsets[1] = 0.0;
    the_params->y_height = 1.0;
    the_params->sample_rate = 1000.0;
    the_params->intpoling = 0;
    the_params->point_diff = 0;
    the_params->ipolj = 0;
    the_params->pointi = 1;
    the_params->degraded = 0;

}

void spratz_default_params(struct spratz_params *params)
{

    params->the_points[0] = 0.0;
    params->the_points[1] = 0.0;
    params->the_offsets[0] = 0.0;
    params->the_offsets[1] = 0.0;
    params->y_height = 1.0;
    params->sample_rate = 1000.0;
    params->intpoling = 0;
    params->point_diff = 0;
    params->ipolj = 0;
    params->pointi = 1;
    params->degraded = 0;

}

void print_params_spratz(struct spratz_params *params)
{

}

// input sind 2 floats in points und 2 floats in offsets
static void gen_rotated_point(int point_id, float *points, float *offsets)
{

    float x_offset = offsets[0];
    float y_offset = offsets[1];

    float tmp_point_0 = 0.0;
    float tmp_point_1 = 0.0;
    // Berechne die erste Hälfte des Spratzes. Der SPratz ist symmetrisch .:|*' oder |:.gj
    // Hier werden also in points die x,y-Werte der Spratz-Basis-Punkte der Ersten Hälfte gespeichert.
    // In der Schleife danach werden dann die Werte gespiegelt in die zweite Hälfte des Spratzes geschrieben.
    if(point_id > 0 && point_id < depth*2){
        if(!(point_id%2)){
            offsets[0] += x_fact*the_params->y_height;
            the_params->y_height *= y_fact;
            points[1] = 0.0;
        }else{
            offsets[1] = the_params->y_height;
            points[1] = offsets[1];
        }
        points[0] = offsets[0];
    }

    // Hier die zwiete Hälfte des Spratzes
    
    if(point_id >= depth*2 && point_id < depth*2*2){
        if(point_id%2){
            if(point_id < depth*2*2-1){
                the_params->y_height *= y_fact_inv;
            }
            points[1] = max_height;
        }else{
            offsets[0] += x_fact*the_params->y_height;
            offsets[1] = the_params->y_height;
            points[1] = max_height-offsets[1];
        }
        points[0] = offsets[0];
    }

    tmp_point_0 = points[0]*rot_0_0 + points[1]*rot_0_1;
    tmp_point_1 = points[0]*rot_1_0 + points[1]*rot_1_1;
    points[0] = tmp_point_0;
    points[1] = tmp_point_1;
    //fprintf(stdout, "Points 1 %f 0 %f Offsets 1 %f 0 %f Depth %i\n", points[1],points[0], offsets[1],offsets[0], the_depth);

}

static void make_points()
{


}

static void rotate_points()
{

}

static void interpolate_points()
{

}

void makeSpratz(const int length, float *buff_L)
{

    in_the_making = 1;
    int counter = 0;
    int ready = 0;
    float max_height_inverse = 1.0;
    if(max_height > 1.0){
        max_height_inverse = 1/max_height;
    }

    while(counter < length){
        if(the_params->pointi == depth*2*2){
            the_params->pointi = 1;
            the_params->the_points[0] = 0.0;
            the_params->the_points[1] = 0.0;
            the_params->the_offsets[0] = 0.0;
            the_params->the_offsets[1] = 0.0;
        }
        //fprintf(stdout, "#counter %i intpol %i\n", counter, the_params->intpoling);
        for(the_params->pointi; the_params->pointi < depth*2*2; ++the_params->pointi){
            if(the_params->intpoling == 0){
                gen_rotated_point(the_params->pointi, the_params->the_points+2, the_params->the_offsets);
                //fprintf(stdout, "Points 3 %f 2 %f 1 %f 0 %f\n", the_params->the_points[3],the_params->the_points[2],the_params->the_points[1],the_params->the_points[0]);
                the_params->point_diff = (int) roundf(the_params->sample_rate*(the_params->the_points[2]-the_params->the_points[0])/quer_length);
                the_params->ipolj = 0;
            }
            if(the_params->point_diff > 0){
                the_params->intpoling = 1;
                for(the_params->ipolj; the_params->ipolj < the_params->point_diff; ++the_params->ipolj){
                    if(max_height > 1.0){
                        buff_L[counter] = (float) max_height_inverse*(the_params->the_points[1] + the_params->ipolj*(the_params->the_points[3]-the_params->the_points[1])/the_params->point_diff);
                    }else{
                        buff_L[counter] = (float) the_params->the_points[1] + the_params->ipolj*(the_params->the_points[3]-the_params->the_points[1])/the_params->point_diff;
                    }

                    //fprintf(stdout, "%i %f %i %i\n", counter, buff_L[counter], length, the_params->pointi);
                    ++counter;
                    if(counter == length){
                        ready = 1;
                        break;
                    }
                }
            }
            if(ready == 1){
                break;
            }else{
                the_params->intpoling = 0;
                the_params->the_points[0] = the_params->the_points[2];
                the_params->the_points[1] = the_params->the_points[3];
            }
            //fprintf(stdout, "%i) %f (%i) (%f) %f\n",i, port, (int) roundf(the_params->sample_rate*port), the_params->sample_rate*port, the_params->the_points[2*i+1]);
            //zum += (int) roundf(the_params->sample_rate*port);
        }
        if(ready == 1){
            ++the_params->ipolj;
            ready = 0;
            //fprintf(stdout, "counter %i\n", counter);
        }
    }
    if(vals_need_update == 1){
        fprintf(stdout, "IN THE MAKING\n");
        update_from_shadow();
        vals_need_update = 0;
    }
    in_the_making = 0;
}

void spratz_set_x_fact(const float the_fact)
{

    if(in_the_making == 0){
        x_fact = the_fact;
    }else{
        vals_need_update = 1;
        spratz_shadow_vals[0] = the_fact;
        spratz_shadow_vals_changed[0] = 1;
    }
    degrade_id = ++degrade_id%32768;

}
void spratz_set_y_fact(const float the_fact)
{

    if(in_the_making == 0){
        y_fact = the_fact;
    }else{
        vals_need_update = 1;
        spratz_shadow_vals[1] = the_fact;
        spratz_shadow_vals_changed[1] = 1;
    }
    degrade_id = ++degrade_id%32768;

}
void spratz_set_depth(const int value)
{

    if(value < 1){
       return;
    }
    if(in_the_making == 0){
        depth = value;
    }else{
        vals_need_update = 1;
        spratz_shadow_vals[2] = value;
        spratz_shadow_vals_changed[2] = 1;
    }
    degrade_id = ++degrade_id%32768;

}
void spratz_set_samplerate(const float value)
{

    if(in_the_making == 0){
        spratz_sample_rate = value;
    }else{
        vals_need_update = 1;
        spratz_shadow_vals[3] = value;
        spratz_shadow_vals_changed[3] = 1;
    }
    degrade_id = ++degrade_id%32768;

}
float spratz_get_x_fact()
{

    return x_fact;

}
float spratz_get_y_fact()
{

    return y_fact;

}

int spratz_get_depth()
{

    return depth;

}

float spratz_get_samplerate()
{

    return spratz_sample_rate;

}

static void update_from_shadow()
{

    if(spratz_shadow_vals_changed[0] == 1){
        x_fact = spratz_shadow_vals[0];
        spratz_shadow_vals_changed[0] == 0;
    }

    if(spratz_shadow_vals_changed[1] == 1){
        y_fact = spratz_shadow_vals[1];
        spratz_shadow_vals_changed[1] == 0;
    }
    if(spratz_shadow_vals_changed[2] == 1){
        depth = spratz_shadow_vals[2];
        spratz_shadow_vals_changed[2] == 0;
    }
    if(spratz_shadow_vals_changed[3] == 1){
        spratz_sample_rate = spratz_shadow_vals[3];
        spratz_shadow_vals_changed[3] == 0;
    }

}

void spratz_set_self_control(const int value)
{

    self_control = value;

}
int spratz_get_self_control()
{

    return self_control;

}

