// Copyright 2014-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#ifndef ESCWYP_SPRATZ_H
#define ESCWYP_SPRATZ_H
#include "escwyp_spratz_params.h"

static int depth = 9;
static float init_size = 1.0;
static float x_length = 1.0;
static float x_fact = 0.475;
static float y_fact = 0.793;
static float y_fact_inv;
static float max_height = 1.0;
static float spratz_sample_rate = 64.0;
static float spratz_sample_width = 1.0;
static float spratz_shadow_vals[4];
static int spratz_shadow_vals_changed[4];
static int vals_need_update = 0;
static int in_the_making = 0;
static int degrade_id = 0;

// Für die Rotation des Spratz hier den Winkel etc. ausrechnen
static float verhaeltnis;
static float winkerl;
static float rot_0_0;
static float rot_0_1;
static float rot_1_0;
static float rot_1_1;
static float quer_length;

// depth is the number of squares in the first half
// times 2 because of 2 points per square
// times 2 because of 2 coordinates per point
// times 2 because of the second half
// the_points = [square0_point0_coord0, square0_point0_coord1,
//               square0_point1_coord0, square0_point1_coord1,
//               square1_point0_coord0, square1_point0_coord1,
//               square1_point1_coord0, square1_point1_coord1, ...]

static float quer_length = 1.0;

// These go into params
static float the_points[4];
static float the_offsets[2];
static float y_height = 1.0;
static int intpoling = 0;
static int point_diff = 0;
static int ipolj = 0;
static int pointi = 1;
//----------------

static int self_control = 0;

void spratz_default_params(struct spratz_params *the_params);
void print_params_spratz(struct spratz_params *the_params);

static struct spratz_params default_params;
static struct spratz_params *the_params = &default_params;

static void gen_rotated_point(int point_id, float *points, float *offsets);
static void make_points();
static void rotate_points();
static void interpolate_points();
static void update_from_shadow();

void spratz_refresh_parameters();
void spratz_set_params(struct spratz_params *params);
void spratz_set_freq(const float freq);
void spratz_set_freq_from_klavier(const float freq, const float freq_low, const float freq_high);
void spratz_init_values();

void spratz_set_x_fact(const float the_fact);
void spratz_set_y_fact(const float the_fact);
void spratz_set_depth(const int value);
void spratz_set_samplerate(const float value);
float spratz_get_x_fact();
float spratz_get_y_fact();
int spratz_get_depth();
float spratz_get_samplerate();

void spratz_constructor();
void makeSpratz(const int length, float *buff_L);
void spratz_set_self_control(const int value);
int spratz_get_self_control();

#endif
